"""
MIT LICENSE

Copyright (C) 2023 Murilo Marques Marinho (www.murilomarinho.info)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import os
import argparse
import pathlib
from notros2.pkg.create._commons import \
    _check_common_inputs, \
    _parse_ament_dependencies_for_cmake

from notros2.pkg.create.ament_cmake.templates.library import cpp, hpp, cmake, cmake_dependencies


def get_cmake_dependencies_for_library(path: pathlib.Path, args: argparse.Namespace) -> str:
    _check_common_inputs(path, args)

    return cmake_dependencies.get_source(args, context={})


def create_hpp_for_library(path: pathlib.Path, args: argparse.Namespace) -> None:
    _check_common_inputs(path, args)

    include_path = path / pathlib.Path('include')
    package_include_path = include_path / pathlib.Path(f'{args.package_name}')
    if not os.path.isdir(include_path):
        os.mkdir(include_path)
        if not os.path.isdir(package_include_path):
            os.mkdir(package_include_path)

    if not args.interfaces_only:
        if not vars(args)['has_library']:
            print(f"Creating sample placeholder for library hpp ...")
            with open(package_include_path / pathlib.Path('.placeholder'), 'w+') as _:
                pass
        else:
            print(f"Creating sample library hpp ...")
            library_hpp_str = hpp.get_source(args, context={})
            with open(package_include_path / pathlib.Path('sample_class.hpp'), 'w+') as library_hpp_file:
                library_hpp_file.write(library_hpp_str)


def create_cpp_for_library(path: pathlib.Path, args: argparse.Namespace) -> None:
    _check_common_inputs(path, args)

    if vars(args)['has_library'] is False:
        return
    library_source_path = path / pathlib.Path('src')
    if not os.path.isdir(library_source_path):
        os.mkdir(library_source_path)

    print(f"Creating sample library cpp ...")

    library_cpp_str = cpp.get_source(args, context={})
    with open(library_source_path / pathlib.Path(f'sample_class.cpp'), 'w+') as library_cpp_file:
        library_cpp_file.write(library_cpp_str)


def get_cmake_for_library(path: pathlib.Path, args: argparse.Namespace):
    _check_common_inputs(path, args)

    if not args.has_library:
        return ""

    print(f"Adding CMakeLists.txt directive for library ...")

    ament_dependencies_str = _parse_ament_dependencies_for_cmake(args)

    context = {'ament_dependencies_str': ament_dependencies_str}
    add_lib_str = cmake.get_source(args, context)

    return add_lib_str



"""
MIT LICENSE

Copyright (C) 2023 Murilo Marques Marinho (www.murilomarinho.info)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import argparse
import sys
import pathlib

from notros2._configuration_file_manager import make_or_get_default_user_configuration
from notros2.pkg.create import create_package_template
from notros2._configuration_file_manager import save_user_configuration


def main() -> None:
    try:

        parser = argparse.ArgumentParser(prog='notros2',
                                         description='ROS2 utilities.',
                                         epilog='This is a package template generator based on the tutorials at '
                                                'https://ros2-tutorial.readthedocs.io/en/latest/',
                                         )
        subparsers = parser.add_subparsers(help='sub-command help', required=True, dest='command')

        # Get current configuration
        config = make_or_get_default_user_configuration()

        # "pkg"
        parser_pkg = subparsers.add_parser('pkg', help='utilities for ROS2 packages.')
        subparsers_pkg = parser_pkg.add_subparsers(help='sub-command help', required=True, dest='subcommand')

        # "pkg set"
        parser_set = subparsers_pkg.add_parser('set', help='set ROS2 default package template info.')
        set_arguments: list[str] = ['mantainer-name', 'mantainer-email', 'license']

        for argument in set_arguments:
            current_value_of_argument = config[argument.replace("-", "_")]
            parser_set.add_argument(f'--{argument}',
                                    action='store',
                                    type=str,
                                    help=f"sets the default {argument}. Currently '{current_value_of_argument}'"
                                    )

        # "pkg create"
        parser_pkg_create = subparsers_pkg.add_parser('create', help='create ROS2 package templates.')
        parser_pkg_create.add_argument('package_name', type=str)
        parser_pkg_create.add_argument('build_type',
                                       type=str,
                                       choices=['ament_cmake',
                                                'ament_python',
                                                'interfaces_only'])

        parser_pkg_create.add_argument('--ament-dependencies',
                                       type=str,
                                       help="The ament dependencies, e.g. ROS2 packages such as `rclcpp` and `rclpy`.",
                                       action='store',
                                       nargs='+')

        parser_pkg_create.add_argument('--add-nodes',
                                       type=str,
                                       help="A sequence of names of sample nodes, e.g. `my_sample_node`.",
                                       action='store',
                                       nargs='+')

        parser_pkg_create.add_argument('--has-library',
                                       help="add this flag if the package should contain a library.",
                                       action="store_true",
                                       default=False
                                       )

        if len(sys.argv) > 1:
            args = parser.parse_args(sys.argv[1:])
        else:
            parser.parse_args(["-h"])
            return

        if args.command == "pkg":
            if args.subcommand == "create":
                if args.build_type == "interfaces_only":
                    # The user input 'interfaces_only' is just to get us until here, so we fix to the
                    # correct build type from here onward. In ROS2, interface only packages must be
                    # build with ament_cmake.
                    args.build_type = "ament_cmake"
                    args.interfaces_only = True
                else:
                    args.interfaces_only = False
                create_package_template(pathlib.Path("."), config, args)
            elif args.subcommand == "set":
                if len(sys.argv) == 3:
                    parser.parse_args(["pkg", "set", "-h"])
                    return
                arg_vars = vars(args)
                config_changed = False
                for argument in set_arguments:
                    argument_for_dict = argument.replace("-", "_")
                    if argument_for_dict in arg_vars:
                        if arg_vars[argument_for_dict] is not None:
                            config[argument_for_dict] = arg_vars[argument_for_dict]
                            config_changed = True
                if config_changed:
                    print("Saving new user configuration with values:")
                    print(config)
                    save_user_configuration(config)

    except KeyboardInterrupt:
        pass
    except Exception as e:
        print("Error ", e)


if __name__ == "__main__":
    main()



"""
MIT LICENSE

Copyright (C) 2023 Murilo Marques Marinho (www.murilomarinho.info)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import os
import argparse
import pathlib

import notros2.pkg.create.ament_cmake as ac
import notros2.pkg.create.ament_python as ap
from ._package_xml_template import create_package_xml_from_template


def create_package_template(path: pathlib.Path, config: dict, args: argparse.Namespace) -> None:
    """
    Creates the package template based on the input arguments and the configuration file.
    :param path: the path for where the package should be made.
    :param config: the configuration dictionary.
    :param args: the parsed commandline arguments.
    :return: None
    :except: raises ValueErrors when an input is invalid.
    """
    package_path = path / pathlib.Path(args.package_name)
    if not os.path.isdir(package_path):
        os.mkdir(package_path)
    else:
        raise ValueError(f"The directory {args.package_name} already exists, choose another name for your package.")

    if args.build_type == "ament_cmake":
        if args.interfaces_only:
            print(f"Creating interfaces_only (ament_cmake) package {args.package_name} ... ")
            create_package_xml_from_template(package_path, config, args)
            ac.create_cmakelists_txt_from_template(package_path, args)
            ac.create_msg_for_interfaces_only(package_path, args)
            ac.create_srv_for_interfaces_only(package_path, args)

            if vars(args)['add_nodes'] is not None:
                print("WARNING: --add-nodes is ignored with build-type=interfaces-only.")
            if vars(args)['has_library']:
                print("WARNING: --has-library is ignored with build-type=interfaces-only.")
        else:
            print(f"Creating ament_cmake package {args.package_name} ... ")
            create_package_xml_from_template(package_path, config, args)
            ac.create_cmakelists_txt_from_template(package_path, args)
            ac.create_cpp_for_nodes(package_path, args)
            ac.create_hpp_for_nodes(package_path, args)
            ac.create_cpp_for_library(package_path, args)
            ac.create_hpp_for_library(package_path, args)

    elif args.build_type == "ament_python":
        print(f"Creating ament_python package {args.package_name} ... ")
        create_package_xml_from_template(package_path, config, args)
        ap.create_setup_py(package_path, config, args)
        ap.create_setup_cfg(package_path, args)
        ap.create_resource_file(package_path, args)
        ap.create_module(package_path, args)
        ap.create_py_for_nodes(package_path, args)
        ap.create_py_for_library(package_path, args)
    else:
        raise NotImplementedError("How did you get here?")



import argparse
import os
import pathlib


def _check_common_inputs(path: pathlib.Path, args: argparse.Namespace):
    if path is None:
        raise ValueError('Path must not be None.')
    if args is None:
        raise ValueError('args must not be None.')
    if not os.path.isdir(path):
        raise ValueError(f'the path={path} does not exist.')


def _parse_ament_dependencies_for_cmake(args: argparse.Namespace) -> str:
    ament_dependencies_str = ""
    if args.ament_dependencies is not None:
        for ament_dependency in args.ament_dependencies:
            ament_dependencies_str = ament_dependencies_str + f'    {ament_dependency}\n'
    return ament_dependencies_str


"""
MIT LICENSE

Copyright (C) 2023 Murilo Marques Marinho (www.murilomarinho.info)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import os
import json
from pathlib import Path


def make_or_get_default_user_configuration() -> dict:
    """
    Gets the contents of the configuration file whose path is defined by
    get_configuration_file_path()
    or creates a new default user configuration at that same location.
    :return: a dict equivalent to the contents of the configuration file.
    """
    if not os.path.isfile(get_configuration_file_path()):
        default_user_configuration = {
            "mantainer_name": 'TODO',
            "mantainer_email": 'TODO@TO.DO',
            "license": 'TODO'
        }

        save_user_configuration(default_user_configuration)

    return get_user_configuration()


def get_configuration_file_path() -> Path:
    """
    Obtains the configuration file path, defined by
    ``~/.notros2/config.json``.
    The directory is defined in
    make_or_get_user_configuration_directory()
    while the filename is defined herein.
    :return: a pathlib.Path object.
    """
    return make_or_get_user_configuration_directory() / Path("config.json")


def get_user_configuration() -> dict:
    """
    Gets the contents of the configuration file whose path is defined by
    get_configuration_file_path()
    :return: a dict with the contents of the file.
    """
    with open(get_configuration_file_path(), 'r') as configuration_file:
        config = json.load(configuration_file)
    return config


def save_user_configuration(config: dict) -> None:
    """
    Saves a dict into the configuration file defined by
    get_configuration_file_path()
    :param config: The dict to be saved.
    :return: None.
    """
    with open(get_configuration_file_path(), 'w') as configuration_file:
        configuration_file.write(json.dumps(config))


def make_or_get_user_configuration_directory() -> Path:
    """
    Obtains or creates the user configuration directory, namely
    ``~/.notros2``.
    :return: a pathlib.Path to the folder.
    """
    user_home = os.path.expanduser("~")
    user_config_directory = user_home / Path('.notros2')

    if not os.path.isdir(user_config_directory):
        os.mkdir(user_config_directory)

    return user_config_directory



"""
MIT LICENSE

Copyright (C) 2023 Murilo Marques Marinho (www.murilomarinho.info)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import os
import argparse
import pathlib
from notros2.pkg.create._commons import \
    _check_common_inputs, \
    _parse_ament_dependencies_for_cmake

from notros2.pkg.create.ament_cmake.templates.simple_node import cpp, main_cpp, hpp, cmake


def create_cpp_for_nodes(path: pathlib.Path, args: argparse.Namespace) -> None:
    _check_common_inputs(path, args)

    if vars(args)['add_nodes'] is None:
        return
    cpp_source_path = path / pathlib.Path('src')
    if not os.path.isdir(cpp_source_path):
        os.mkdir(cpp_source_path)

    for node_name in vars(args)['add_nodes']:
        print(f"Creating {node_name}.cpp ...")
        NodeName = node_name.replace("_", " ").title().replace(" ", "")

        context = {
            'node_name': node_name,
            'NodeName': NodeName
        }
        node_cpp_str = cpp.get_source(args, context)

        with open(cpp_source_path / pathlib.Path(f'{node_name}.cpp'), 'w+') as node_cpp_file:
            node_cpp_file.write(node_cpp_str)

        print(f"Creating {node_name}_main.cpp ...")

        node_cpp_main_str = main_cpp.get_source(args, context)

        with open(cpp_source_path / pathlib.Path(f'{node_name}_main.cpp'), 'w+') as node_cpp_main_file:
            node_cpp_main_file.write(node_cpp_main_str)


def create_hpp_for_nodes(path: pathlib.Path, args: argparse.Namespace) -> None:
    _check_common_inputs(path, args)

    if vars(args)['add_nodes'] is None:
        return
    cpp_source_path = path / pathlib.Path('src')
    if not os.path.isdir(cpp_source_path):
        os.mkdir(cpp_source_path)

    for node_name in vars(args)['add_nodes']:
        print(f"Creating {node_name}.hpp ...")

        NodeName = node_name.replace("_", " ").title().replace(" ", "")

        context = {
            "NodeName": NodeName
        }
        node_hpp_str = hpp.get_source(args, context)

        with open(cpp_source_path / pathlib.Path(f'{node_name}.hpp'), 'w+') as node_hpp_file:
            node_hpp_file.write(node_hpp_str)


def get_cmake_for_nodes(path: pathlib.Path, args: argparse.Namespace) -> str:
    _check_common_inputs(path, args)

    if vars(args)['add_nodes'] is None:
        return ""

    ament_dependencies_str = _parse_ament_dependencies_for_cmake(args)

    add_nodes_str = ""
    for node_name in vars(args)['add_nodes']:
        print(f"Adding CMakeLists.txt directive for {node_name} ...")

        nl = '\n'

        context = {
            "node_name": node_name,
            "ament_dependencies_str": ament_dependencies_str,
            "nl": nl
        }
        add_node_str = cmake.get_source(args, context)

        add_nodes_str = add_nodes_str + add_node_str

    return add_nodes_str



"""
MIT LICENSE

Copyright (C) 2023 Murilo Marques Marinho (www.murilomarinho.info)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import textwrap
import argparse
import pathlib

from notros2.pkg.create._commons import _check_common_inputs
from ._add_node import get_cmake_for_nodes
from ._add_library import get_cmake_for_library, get_cmake_dependencies_for_library
from ._add_message_only import get_cmake_for_interfaces_only


def create_cmakelists_txt_from_template(path: pathlib.Path, args: argparse.Namespace) -> None:
    """
    Creates the CMakeLists.txt for an ament_cmake package.
    :param path: the path to the root folder of the package.
    :param args: the parsed commandline arguments.
    :return: None
    :except: Raises ValueErrors when the inputs are wrong.
    """
    _check_common_inputs(path, args)

    print(f"Creating CMakeLists.txt ... ")

    ament_dependencies_str = ""
    if vars(args)['ament_dependencies'] is not None:
        for ament_dependency in args.ament_dependencies:
            ament_dependencies_str = ament_dependencies_str + f'find_package({ament_dependency} REQUIRED)\n'
        ament_dependencies_str = ament_dependencies_str + '  '

    cmakelists_txt_str = textwrap.dedent(f"""\
cmake_minimum_required(VERSION 3.8)
project({args.package_name})

if(CMAKE_COMPILER_IS_GNUCXX OR CMAKE_CXX_COMPILER_ID MATCHES "Clang")
    add_compile_options(-Wall -Wextra -Wpedantic)
endif()

# find dependencies
find_package(ament_cmake REQUIRED)
{get_cmake_dependencies_for_library(path, args)}
{ament_dependencies_str}\
{get_cmake_for_library(path, args)}\
{get_cmake_for_nodes(path, args)}\
{get_cmake_for_interfaces_only(path, args)}
if(BUILD_TESTING)
    find_package(ament_lint_auto REQUIRED)
    # the following line skips the linter which checks for copyrights
    # comment the line when a copyright and license is added to all source files
    set(ament_cmake_copyright_FOUND TRUE)
    # the following line skips cpplint (only works in a git repo)
    # comment the line when this package is in a git repo and when
    # a copyright and license is added to all source files
    set(ament_cmake_cpplint_FOUND TRUE)
    ament_lint_auto_find_test_dependencies()
endif()

ament_package()\
    """)

    with open(path / pathlib.Path('CMakeLists.txt'), 'w+') as cmakelists_txt_file:
        cmakelists_txt_file.write(cmakelists_txt_str)




from setuptools import setup, find_packages

# Automatic version
from datetime import datetime
version_st = datetime.today().strftime('%y.%m.%d.%H%M%S')

# https://packaging.python.org/en/latest/guides/making-a-pypi-friendly-readme/
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='notros2',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version=version_st,
    packages=find_packages(exclude=['test']),
    install_requires=['setuptools'],
    zip_safe=True,
    maintainer='Murilo M. Marinho',
    maintainer_email='murilomarinho@ieee.org',
    description='notROS2 utilities.',
    license='MIT',
    entry_points={
        'console_scripts': [
            'notros2 = notros2.notros2_script:main'
        ],
    },
)



"""
MIT LICENSE

Copyright (C) 2023 Murilo Marques Marinho (www.murilomarinho.info)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import argparse


def get_source(args: argparse.Namespace, context: dict) -> str:
    return f"""\
//Template from https://ros2-tutorial.readthedocs.io/en/latest/
#include <{args.package_name}/sample_class.hpp>



/**
 * @brief SampleClass::SampleClass the default constructor.
 */
SampleClass::SampleClass():
    a_private_qt_string_("I am a QString"),
    a_private_eigen3_matrix_((Eigen::Matrix2d() << 1,2,3,4).finished())
{{

}}

/**
 * @brief SampleClass::get_a_private_member.
 * @return an int with the value of a_private_member_.
 */
int SampleClass::get_a_private_member() const
{{
    return a_private_member_;
}}

/**
 * @brief SampleClass::set_a_private_member.
 * @param [in] value The new value for a_private_member_.
 */
void SampleClass::set_a_private_member(int value)
{{
    a_private_member_ = value;
}}

/**
 * @brief SampleClass::sum_of_squares.
 * @param [in] a The first number.
 * @param [in] b The second number.
 * @return a*a + 2*a*b + b*b.
 */
double SampleClass::sum_of_squares(const double &a, const double &b)
{{
    return a*a + 2*a*b + b*b;
}}

/**
 * @brief SampleClass::to_string converts a SampleClass to a std::string representation.
 * @return a pretty(-ish) std::string representation of the object.
 */
std::string SampleClass::to_string() const
{{
    std::stringstream ss;
    ss << "Sample_Class:: " << std::endl <<
          "a_private_member_ = "        << std::to_string(a_private_member_) << std::endl <<
          "a_private_qt_string_ = "     << a_private_qt_string_.toStdString() << std::endl <<
          "a_private_eigen3_matrix_ = " << a_private_eigen3_matrix_ << std::endl;
    return ss.str();
}}

/**
 * @brief operator << the stream operator for SampleClass objects.
 * @param [in/out] the std::ostream to be modified.
 * @param [in] sc the SampleClass whose representation is to be streamed.
 * @return the modified os with the added SampleClass string representation.
 * @see SampleClass::to_string().
 */
std::ostream &operator<<(std::ostream &os, const SampleClass &sc)
{{
    return os << sc.to_string();
}}
"""
